#
# START COPYRIGHT-NOTICE: 2002-2003
#
# Copyright 2002-2003 VERITAS Software Corporation.
# All rights reserved.
#
# VERITAS, VERITAS SOFTWARE, the VERITAS logo and all other VERITAS
# product names and slogans are trademarks or registered trademarks
# of VERITAS Software Corporation in the USA and/or other countries.
# Other product names and/or slogans mentioned herein may be trade-
# marks or registered trademarks of their respective companies.
#
# END COPYRIGHT-NOTICE.
#

package ArrayRep;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);

@EXPORT = qw(
);
$VERSION = '1.00';

require 5.000;
my $vcs_home=$ENV{"VCS_HOME"} || "/opt/VRTSvcs";
use ag_i18n_inc;

my $VXCLUSTADM = "/etc/vx/bin/vxclustadm";

sub create_lockfile()
{
	my($self) = shift;
	my($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	my $LockFile = $vcs_home . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	#TBD: if lockfile creation failed
	open(lfh, ">$LockFile");
	close(lfh);
}

sub remove_lockfile()
{
	my($self) = shift;
	my($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	my $LockFile = $vcs_home . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	#TBD: if lockfile removal failed
	if (-f "$LockFile") {
		unlink("$LockFile");
	}
}

sub check_lockfile()
{
	my($self) = shift;
	my($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	my $LockFile = $vcs_home . "/bin/" . $typ . "/.VCS_" . $typ . "_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	#TBD: if lockfile removal failed
	if (-f "$LockFile") {
		return 110;
	} else {
		return 100;
	}
}

sub serialize {
	my($self) = shift;
	my($grp) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Group`);
	my($para) = split('\n', `$vcs_home/bin/hagrp -value $grp Parallel`);
	if ($para != 1) {
		return 0;
	}
	my $ret = "";
	open(INPUT,"$vcs_home/bin/hares -lock $self->{'resname'} 2>&1 |");
	while (my $line = <INPUT>) { chomp($line); $ret = ($ret) ? "$ret\n$line" : $line; }
	close INPUT;
	my $retval = $?;
	if ($retval == 0) {
		return 0;
	}
	my (@lockholder) = split('\s+', $ret);
	VCSAG_LOG_MSG("N", "Resource $self->{'resname'} will wait for onlining instance on node $lockholder[9] to complete", 60015, "$self->{'resname'}", "$lockholder[9]");
	while ($retval) {
		sleep(5);
		$retval = system("$vcs_home/bin/hares -lock $self->{'resname'} > /dev/null 2>&1");
	}
	return 1; # had to wait
}

sub cvm_import {
	my($self) = shift;
	my $nodename = "";
	my ($grp) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Group`);
	my ($par) = split('\n', `$vcs_home/bin/hagrp -value $grp Parallel`);
	if ($par != 1) {
		return 0;
	}
	my ($sup) = split('\n', `$vcs_home/bin/hatype -value CVMVolDg SupportedActions`);
	if (! (($sup =~ /deport/) && ($sup =~ /import/))) {
		return 0;
	}

	my $retval = system("$vcs_home/bin/hares -lock $self->{'resname'} > /dev/null 2>&1");
	while ($retval) {
		sleep(5);
		$retval = system("$vcs_home/bin/hares -lock $self->{'resname'} > /dev/null 2>&1");
	}

	my (@states) = split('\n', `$vcs_home/bin/hares -state $self->{'resname'} -localclus`);
	foreach my $sts (@states) {
		if ($sts =~ /ONLINE/) {
			return 0;
		}
	}
	my $found = 0;
	my (@vxclus) = split('\n', `$VXCLUSTADM nidmap 2>/dev/null`);
	foreach my $v (@vxclus) {
		if ($v =~ /Master/) {
			my (@v1) = split('\s+', $v);
			$nodename = $v1[0];
			$found = 1;
			last;
		}
	}
	if (!$found) {
		return 0;
	}
	my($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	VCSAG_LOG_MSG("N", "resource $self->{'resname'} is its group's first $typ resource in this cluster to online; all CVM Disk Groups used by this group will be imported on all nodes if they have not already been", 9, "$self->{'resname'}", "$typ");
	my $import_failed = 0;
	my (@deps) = split('\n', `$vcs_home/bin/hares -dep $self->{'resname'}`);
	foreach my $d (@deps) {
		if ($d =~ /#Group/) {
			next;
		}
		my (@fields) = split('\s+', $d);
		if ($fields[2] ne $self->{'resname'}) {
			next;
		}
		my ($typ) = split('\n', `$vcs_home/bin/hares -value $fields[1] Type`);
		if ($typ eq "CVMVolDg") {
			my $ret = system("$vcs_home/bin/hares -action $fields[1] import -sys $nodename");
			if ($ret) {
				#TBD: get output from failure
				$import_failed = 1;
			}
		}
	}
	return $import_failed;
}

sub cvm_deport {
	my($self) = shift;
	my $nodename = "";
	my $sleeptime = 0;
	my $ctr = 0;
	my ($grp) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Group`);
	my ($par) = split('\n', `$vcs_home/bin/hagrp -value $grp Parallel`);
	if ($par != 1) {
		return 0;
	}
	my ($sup) = split('\n', `$vcs_home/bin/hatype -value CVMVolDg SupportedActions`);
	if (! (($sup =~ /deport/) && ($sup =~ /import/))) {
		return 0;
	}
        my ($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	my(@fields) = split('\s+', `$vcs_home/bin/hasys -nodeid`);
	my($sys) = $fields[5];

	my $retval = system("$vcs_home/bin/hares -lock $self->{'resname'} > /dev/null 2>&1");
	while ($retval) {
		sleep(5);
		$retval = system("$vcs_home/bin/hares -lock $self->{'resname'} > /dev/null 2>&1");
	}

	my (@states) = split('\n', `$vcs_home/bin/hares -state $self->{'resname'} -localclus`);
	foreach my $st (@states) {
		my (@fields) = split('\s+', $st);
		if (($fields[3] eq "ONLINE") && ($fields[2] ne $sys)) {
			return 0;
		}
	}
	my $found = 0;
	my (@vxclus) = split('\n', `$VXCLUSTADM nidmap 2>/dev/null`);
	foreach my $v (@vxclus) {
		if ($v =~ /Master/) {
			my (@v1) = split('\s+', $v);
			$nodename = $v1[0];
			$found = 1;
			last;
		}
	}
	if (!$found) {
		return 0;
	}
	my($typ) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Type`);
	VCSAG_LOG_MSG("N", "resource $self->{'resname'} is its group's last $typ resource in this cluster to offline; all CVM Disk Groups used by this group will be deported on all nodes if they have not already been", 10, "$self->{'resname'}", "$typ");
	my $deport_failed = 0;
	my (@deps) = split('\n', `$vcs_home/bin/hares -dep $self->{'resname'}`);
	foreach my $d (@deps) {
		if ($d =~ /#Group/) {
			next;
		}
		my (@fields) = split('\s+', $d);
		if ($fields[2] ne $self->{'resname'}) {
			next;
		}
		my ($typ) = split('\n', `$vcs_home/bin/hares -value $fields[1] Type`);
		if ($typ eq "CVMVolDg") {
			my $ret = system("$vcs_home/bin/hares -action $fields[1] deport -sys $nodename");
			if ($ret) {
				#TBD: get output from failure
				$deport_failed = 1;
			}
		}
	}
	return $deport_failed;
}

		
